Documentation
¶
Index ¶
- type Array
- func (a Array) Array(idx int) Array
- func (a Array) Bool(idx int) (bool, error)
- func (a Array) Bytes() ([]byte, error)
- func (a Array) Err() error
- func (a Array) Float(idx int) (float64, error)
- func (a Array) Inner() ([]interface{}, error)
- func (a Array) Len() (int, error)
- func (a Array) Map(idx int) Map
- func (a Array) String(idx int) (string, error)
- type Map
- func (m Map) Array(key string) Array
- func (m Map) Bool(key string) (bool, error)
- func (m Map) Bytes() ([]byte, error)
- func (m Map) Err() error
- func (m Map) Float(key string) (float64, error)
- func (m Map) Has(key string) (bool, error)
- func (m Map) Inner() (map[string]interface{}, error)
- func (m Map) Map(key string) Map
- func (m Map) String(key string) (string, error)
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Array ¶
type Array struct {
// contains filtered or unexported fields
}
Array provides methods to either navigate through the content of a JSON array or propagate any error that has occurred
func ArrayFromBytes ¶
ArrayFromBytes creates an Array from a []byte
func ArrayFromReader ¶
ArrayFromReader creates an Array from an io.Reader
func (Array) Array ¶
Array returns the array found at index `idx` in the array; errors will be propagated
func (Array) Bool ¶
Bool returns the value of a bool at index `idx` in the array, or a propagated error
func (Array) Bytes ¶
Bytes returns the serialized value into a slice of bytes, or a propagated error
func (Array) Float ¶
Float returns the value of a float at index `idx` in the array, or a propagated error
func (Array) Inner ¶ added in v0.3.0
Inner returns the `[]interface{}` which this `Array` represents, or a propagated error
type Map ¶
type Map struct {
// contains filtered or unexported fields
}
Map provides methods to either navigate through the content of a JSON object or propagate any error that has occurred
func MapFromBytes ¶
MapFromBytes creates a Map from a []byte
func MapFromReader ¶
MapFromReader creates a Map from an io.Reader
func (Map) Float ¶
Float returns the value of a float at key `key` in the map, or a propagated error
func (Map) Inner ¶
Inner returns the `map[string]interface{}` which this `Map` represents, or a propagated error